﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;

namespace CFP2API
{
    public enum DEVICETYPE
    {
        ML_CFP2,
        ML_CFP2_FTDI
    };

    public struct FWInfo
    {
        public int FWDeviceID;
        public int FWRev;
        public string DateAndTime;
    }
    enum I2CRate
    {
        _100K_ = 0x00,
        _400K_ = 0x01,
        _10K_ = 0x02,
        _50K_ = 0x03,
        _200K_ = 0x04,
        _300K_ = 0x05,
    };
    public class Wrapper
    {
        private IntPtr Instance;
        const string DLLPath = @"CFP2API.dll";

        #region DLL Imports
        [DllImport(DLLPath, EntryPoint = "mlcfp2_enumerateDevices", CallingConvention = CallingConvention.StdCall)]
        private static extern int apiEnumerateDevices(DEVICETYPE type, UInt16[] Instances, byte[] serialNum);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_createInstance", CallingConvention = CallingConvention.StdCall)]
        private static extern IntPtr apiCreateCFP2Wrapper(DEVICETYPE type, UInt16 instNum);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_connect", CallingConvention = CallingConvention.StdCall)]
        private static extern bool apiConnect(IntPtr Instance);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_disconnect", CallingConvention = CallingConvention.StdCall)]
        private static extern bool apiDisconnect(IntPtr Instance);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_ping", CallingConvention = CallingConvention.StdCall)]
        [return: MarshalAs(UnmanagedType.I1)]
        private static extern bool apiPing(IntPtr Instance, byte[] inBuff);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_eepromRead", CallingConvention = CallingConvention.StdCall)]
        [return: MarshalAs(UnmanagedType.I1)]
        private static extern bool apiReadEEPROM(IntPtr Instance, UInt32 address, ref byte Value);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_eepromWrite", CallingConvention = CallingConvention.StdCall)]
        [return: MarshalAs(UnmanagedType.I1)]
        private static extern bool apiWriteEEPROM(IntPtr Instance, UInt32 address, byte Value);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_resetMCU", CallingConvention = CallingConvention.StdCall)]
        [return: MarshalAs(UnmanagedType.I1)]
        private static extern bool apiResetMCU(IntPtr Instance);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_resetMCUBoot", CallingConvention = CallingConvention.StdCall)]
        [return: MarshalAs(UnmanagedType.I1)]
        private static extern bool apiResetMCUBoot(IntPtr Instance);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_setPort", CallingConvention = CallingConvention.StdCall)]
        [return: MarshalAs(UnmanagedType.I1)]
        private static extern bool apiSetPort(IntPtr Instance, byte t_Port, byte Value);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_getPort", CallingConvention = CallingConvention.StdCall)]
        [return: MarshalAs(UnmanagedType.I1)]
        private static extern bool apiGetPort(IntPtr Instance, byte t_Port, ref byte Value);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_setVendorIO", CallingConvention = CallingConvention.StdCall)]
        [return: MarshalAs(UnmanagedType.I1)]
        private static extern bool apiSetVendorIO(IntPtr Instance, byte Value);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_enableCurrentSense", CallingConvention = CallingConvention.StdCall)]
        [return: MarshalAs(UnmanagedType.I1)]
        private static extern bool apiEnableCurrentSense(IntPtr Instance, UInt16 threshold, UInt16 delay);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_getCurrentSense", CallingConvention = CallingConvention.StdCall)]
        [return: MarshalAs(UnmanagedType.I1)]
        private static extern bool apiGetCurrentSense(IntPtr Instance, UInt16 RegisterAddress, ref byte Data);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_currentMonitor", CallingConvention = CallingConvention.StdCall)]
        [return: MarshalAs(UnmanagedType.I1)]
        private static extern bool apiCurrentMonitor(IntPtr Instance, ref UInt16 Data);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_setMDIO", CallingConvention = CallingConvention.StdCall)]
        [return: MarshalAs(UnmanagedType.I1)]
        private static extern bool apiSetMDIO(IntPtr Instance, byte DeviceAddress, UInt16 RegisterAddress, UInt16 Data);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_getMDIO", CallingConvention = CallingConvention.StdCall)]
        [return: MarshalAs(UnmanagedType.I1)]
        private static extern bool apiGetMDIO(IntPtr Instance, byte DeviceAddress, UInt16 RegisterAddress, ref UInt16 Data);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_getVCC", CallingConvention = CallingConvention.StdCall)]
        [return: MarshalAs(UnmanagedType.I1)]
        private static extern bool apiGetVCC(IntPtr Instance, ref double Data);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_getTemperature", CallingConvention = CallingConvention.StdCall)]
        [return: MarshalAs(UnmanagedType.I1)]
        private static extern bool apiGetTemperature(IntPtr Instance, int select, ref double Data);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_p3v3_FTDIEnable", CallingConvention = CallingConvention.StdCall)]
        [return: MarshalAs(UnmanagedType.I1)]
        private static extern bool apiFTDIEnable(IntPtr Instance, bool isEnabled);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_setPower", CallingConvention = CallingConvention.StdCall)]
        [return: MarshalAs(UnmanagedType.I1)]
        private static extern bool apiSetPower(IntPtr Instance, byte rate);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_getSN", CallingConvention = CallingConvention.StdCall)]
        private static extern bool apiGet_SN(IntPtr Instance, byte[] SN);

        [DllImport(DLLPath, EntryPoint = "mlcfp2_setSN", CallingConvention = CallingConvention.StdCall)]
        private static extern bool apiSet_SN(IntPtr Instance, byte[] SN, int SNsize);
        #endregion

        public Wrapper(DEVICETYPE devType, UInt16 instNum)
        {
            Instance = apiCreateCFP2Wrapper(devType, instNum);
        }
        public static Dictionary<UInt16, byte> EnumerateDevices(DEVICETYPE devType)
        {
            Dictionary<UInt16, byte> Devices = new Dictionary<UInt16, byte>();
            UInt16[] instances = new UInt16[32];
            byte[] serialNum = new byte[32];
            int instCount = 0;
            instCount = apiEnumerateDevices(devType, instances, serialNum);
            for (int i = 0;i < instCount; i++)
            {
               Devices.Add(instances[i], serialNum[i]);
            }
            return Devices;
        }
        public bool Connect()
        {
            return apiConnect(Instance);
        }
        public bool Disconnect()
        {
            return apiDisconnect(Instance);
        }
        public bool Ping(UInt16 ClassInstance, ref FWInfo PingInfo)
        {
            bool rc = false;
            try
            {
                byte[] inBuff = new byte[20];
                rc = apiPing(Instance, inBuff);
                if (rc == true)
                {
                    PingInfo.FWDeviceID = inBuff[0];
                    PingInfo.FWRev = inBuff[1];
                    for (int i = 2; i <= 19; i++)
                        PingInfo.DateAndTime += (char)inBuff[i];
                }
            }
            catch
            {
            }
            return rc;
        }
        public bool ReadEEPROM(UInt32 address, ref byte Value)
        {
            return apiReadEEPROM(Instance, address, ref Value);
        }
        public bool WriteEEPROM(UInt32 address, byte Value)
        {
            return apiWriteEEPROM(Instance, address, Value);
        }
        public bool Reset_MCU()
        {
            return apiResetMCU(Instance);
        }
        public bool Reset_MCU_Boot()
        {
            return apiResetMCUBoot(Instance);
        }
        public bool SetPort(byte t_Port, byte port_value)
        {
            return apiSetPort(Instance, t_Port, port_value);
        }
        public bool GetPort(byte t_Port, ref byte value)
        {
            return apiGetPort(Instance, t_Port, ref value);
        }
        public bool SetVendor_IO(byte Value)
        {
            return apiSetVendorIO(Instance, Value);
        }
        public bool Current_Monitor(ref UInt16 Value)
        {
            return apiCurrentMonitor(Instance, ref Value);
        }
        public bool Set_MDIO(byte deviceAddress, UInt16 register, UInt16 data)
        {
            return apiSetMDIO(Instance, deviceAddress, register, data);
        }
        public bool Get_MDIO(byte deviceAddress, UInt16 register, ref UInt16 data)
        {
            return apiGetMDIO(Instance, deviceAddress, register, ref data);
        }

        public bool Get_VCC(ref double Data)
        {
            return apiGetVCC(Instance, ref Data);
        }

        public bool Get_Temperature(int select, ref double Data)
        {
            return apiGetTemperature(Instance, select, ref Data);
        }

        public bool FTDI_Enable(bool Enable)
        {
            return apiFTDIEnable(Instance, Enable);
        }

        public bool Set_Power(byte rate)
        {
            return apiSetPower(Instance, rate);
        }

        public bool Get_SN(ref string serialNum)
        {
            bool result = false;
            byte[] SNbytes = new byte[8];
            result = apiGet_SN(Instance, SNbytes);
            for (int i = 0; i < 8; i++)
            {
                serialNum += (char)SNbytes[i];
            }
            serialNum = serialNum.TrimEnd('\0');
            return result;
        }

        public bool Set_SN(string serialNum)
        {
            byte[] SNbytes;
            int SNsize = serialNum.Length;
            SNbytes = Encoding.ASCII.GetBytes(serialNum);
            return apiSet_SN(Instance, SNbytes, SNsize);
        }
    }
}
